function START()
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Run this to begin.
%    Builds a Random Forest classifier, using user input, to classify localisations as being accurate/inaccurate.
%    1st GUI asks for user to specify image locations, .csv file location, etc.
%    2nd GUI asks user to label a subset of the localisations, so the Random Forest classifier can train using this.
%    results saved in a .mat file, at location of user's choice.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[ str err] = urlread('http://coxphysics.com/lr/latest_version.txt', 'Timeout', 3)
%Cut off the newline
str = str(1:end-1);
version='1.0'

str + 0.0
version+0.0
strcmp(str, version)


if err == 1 && strcmp(str, version) == 0
	m=msgbox('You are using an old version. A newer version is available from http://coxphysics.com/lr', 'Update version');
end



%various parameter sneed to be set usign the GUI - put some default values in them now
folderName = 'Select folder where images stored, E.G. My_Documents\my_image_folder';
imageExtension = 'Select type of image, E.G. .tiff';
pixelSize = 'E.G. 160';
csvLocation = 'Select .csv file of localisations, E.G. My_Documents\my_image_folder\localisations.csv';
algorithm = 'Specify .csv details (column containing x co-ord, column containing y, units, etc)';
NLabel = 'Select how many samples you want to label, E.G. 900';
numTrees = 'Select how many trees you want the algorithm to train, E.G. 300';
saveLocation = 'Save results in a .mat file, E.G. E.G. My_Documents\my_image_folder\my_results.mat';
csvNumCols = 8;
csvXCol = 2;
csvYCol = 3;
csvFCol = 1;
pixelOffset = 0.5;
localisationsInNm = true;
            

%set some binary flags. These control whether or not the user is free to begin training the forest classifier
gotFolder = false;
gotImageExtension = false;
gotPixelSize = false;
gotCSV = false;
gotAlgo = false;
gotNLabel = false;
gotNumTrees = false;
gotSaveLoc = false;


%standardise dimensions of the GUI
buttonWidth = 100;
buttonHeight = 25;
buttonOffset = 20;
buttonStep = 40;
textWidth = 280;
textHeight = 25;
textOffset = buttonWidth + buttonOffset + 10;

%  Create and then hide the UI as it is being constructed.
scsz = get(0,'ScreenSize');
f = figure('Visible','on','Position',[scsz(1)+3,scsz(4)-400-5,450,400]);

% Construct the components.

%folder location button
hfolderNameV = buttonOffset+8*buttonStep;
hfolderName    = uicontrol('Style','pushbutton','String','folder location','Position', [buttonOffset,hfolderNameV,buttonWidth,buttonHeight], 'Callback',@folderNameButton_Callback);
hfolderNameText  = uicontrol('Style','text','String',folderName,'Position',[textOffset,hfolderNameV,textWidth,textHeight]);

%image extension button
himageExtensionV = buttonOffset+7*buttonStep;
himageExtension = uicontrol('Style','pushbutton','String','image extension','Position', [buttonOffset,himageExtensionV,buttonWidth,buttonHeight], 'Callback', @imageExtensionButton_Callback);
himageExtensionText  = uicontrol('Style','text','String',imageExtension,'Position',[textOffset,himageExtensionV,textWidth,textHeight]);

%pixel size button
hpixelSizeV = buttonOffset+6*buttonStep;
hpixelSize = uicontrol('Style','pushbutton','String','Pixel size','Position', [buttonOffset,hpixelSizeV,buttonWidth,buttonHeight], 'Callback', @pixelSizebutton_Callback);
hpixelSizeText  = uicontrol('Style','text','String',pixelSize,'Position',[textOffset,hpixelSizeV,textWidth,textHeight]);

%.csv location button
hcsvLocationV = buttonOffset+5*buttonStep;
hcsvLocation = uicontrol('Style','pushbutton', 'String','csv file location','Position',[buttonOffset,hcsvLocationV,buttonWidth,buttonHeight],'Callback',@csvLocationButton_Callback);
hcsvLocationText  = uicontrol('Style','text','String',csvLocation,'Position',[textOffset,hcsvLocationV,textWidth,textHeight]);

%.csv layout details button
halgorithmV = buttonOffset+4*buttonStep;
halgorithm = uicontrol('Style','pushbutton','String','csv file details','Position', [buttonOffset,halgorithmV,buttonWidth,buttonHeight], 'Callback', @algorithmButton_Callback);
halgorithmText  = uicontrol('Style','text','String',algorithm,'Position',[textOffset,halgorithmV,textWidth,textHeight]);

%Number to be labelled button
hNLabelV = buttonOffset+3*buttonStep;
hNLabel = uicontrol('Style','pushbutton','String','Num to label','Position', [buttonOffset, hNLabelV, buttonWidth,buttonHeight], 'Callback', @NLabelbutton_Callback);
hNLabelText  = uicontrol('Style','text','String',NLabel,'Position',[textOffset,hNLabelV, textWidth,textHeight]);

%number of trees in the forest button
htreeV = buttonOffset+2*buttonStep;
htree = uicontrol('Style','pushbutton','String','Num of trees','Position', [buttonOffset, htreeV, buttonWidth,buttonHeight], 'Callback', @Ntreebutton_Callback);
htreeText  = uicontrol('Style','text','String',numTrees,'Position',[textOffset,htreeV, textWidth,textHeight]);

%save location button
hsaveV = buttonOffset+1*buttonStep;
hsave = uicontrol('Style','pushbutton', 'String','save results','Position',[buttonOffset,hsaveV,buttonWidth,buttonHeight],'Callback',@saveButton_Callback);
hsaveText  = uicontrol('Style','text','String',saveLocation,'Position',[textOffset,hsaveV,textWidth,textHeight]);

%offset for button which appears when we're finished
hFinishedV = buttonOffset;



%align components
align([hfolderName, himageExtension, hpixelSize, hcsvLocation, halgorithm, hNLabel],'Center','None');

% Initialize the UI.
% Change units to normalized so components resize automatically.
set(f,'Units', 'normalized');
set(hfolderName,'Units', 'normalized');
set(himageExtension,'Units', 'normalized');
set(hpixelSize,'Units', 'normalized');
set(hcsvLocation,'Units', 'normalized');
set(halgorithm,'Units', 'normalized');
set(hNLabel,'Units', 'normalized');
set(htree,'Units', 'normalized');
set(hsave,'Units', 'normalized');
set(hfolderNameText,'Units', 'normalized');
set(himageExtensionText,'Units', 'normalized');
set(hpixelSizeText,'Units', 'normalized');
set(hcsvLocationText,'Units', 'normalized');
set(halgorithmText,'Units', 'normalized');
set(hNLabelText,'Units', 'normalized');
set(htreeText,'Units', 'normalized');
set(hsaveText,'Units', 'normalized');




% Assign the a name to appear in the window title.
set(f,'Name', 'Initialise folders, etc');



%  Pop-up menu callback. Read the pop-up menu Value property to
%  determine which item is currently displayed and make it the
%  current data. This callback automatically has access to 
%  current_data because this function is nested at a lower level.



%callback for folder name button
function folderNameButton_Callback(source,eventdata)

	madeSelection = false;
	while not(madeSelection),
		disp('Pick folder containing the raw images');
		newfolderName = uigetdir;
		if newfolderName == 0,
			break;
		end
		folderName = newfolderName;
		madeSelection = true;
		gotFolder = true;
	end

	run_update();

end

%callback for image extension button
function imageExtensionButton_Callback(source,eventdata)


	madeSelection = false;
	while not(madeSelection),
		disp('Pick file extension');
		imageTypes = {'.tiff', '.TIFF', '.tif', '.TIF', '.png', '.jpg', '.bmp', '.gif'};
		[s,v] = listdlg('PromptString',['Select an image file extension: it should match the extension of the image files in ', folderName], 'SelectionMode','single', 'ListString',imageTypes);
		if v==0,
			break;
		end
		imageExtension = ['*', imageTypes{s}];

		gotImageExtension = true;
		madeSelection = true;


	end
	run_update();

end

%callback for number of trees button
function Ntreebutton_Callback(source,eventdata)
	madeSelection = false;
	while not(madeSelection),
		
		prompt = {'Enter number of trees'};
		dlg_title = 'Set number of trees';
		num_lines = 1;
		if gotNumTrees,
			defAns = {num2str(numTrees)};
		else
			defAns = {'300'};
		end
		answer = inputdlg(prompt,dlg_title,num_lines,defAns);

		if length(answer)==0,
			break;
		end

		if isempty(str2num(answer{1})),
			h = errordlg('number of trees must be a number');
			uiwait(h);
		else
			madeSelection = true;
		end

		if madeSelection,

			numTrees = str2num(answer{1});

			if pixelSize < 0,
				h = errordlg('Number of trees must be greater than 0');
				uiwait(h);	
				madeSelection = false;
			else
				gotNumTrees = true;
				madeSelection = true;
			end
		end
	end
	run_update();

end

%callback for pixel size button
function pixelSizebutton_Callback(source,eventdata)

	madeSelection = false;
	while not(madeSelection),
		disp('Set the pixel size');
		prompt = {'Enter pixel size of your images:'};
		dlg_title = 'Set pixel size in nm';
		num_lines = 1;
		defAns = {'160'};
		answer = inputdlg(prompt,dlg_title,num_lines,defAns);

		if length(answer)==0,
			break;
		end

		if isempty(str2num(answer{1})),
			h = errordlg('Pixel size must be a number');
			uiwait(h);
		else
			madeSelection = true;
		end

		if madeSelection,

			pixelSize = str2num(answer{1});

			if pixelSize < 0,
				h = errordlg('Pixels cant have a size less than 0');
				uiwait(h);	
				madeSelection = false;
			else
				gotPixelSize = true;
				madeSelection = true;
			end
		end
	end
	run_update();

end



%callback for csv location button
function csvLocationButton_Callback(source,eventdata)

	madeSelection = false;
	while not(madeSelection),
		disp('Pick .csv file containing the localisations');

		if gotFolder,
			startFolder = folderName;
		else
			startFolder = pwd;
		end

		[csvLocation,PathName,FilterIndex] = uigetfile([startFolder, '/*.csv']);
		if FilterIndex==0,
			break;
		end
		csvLocation = [PathName,csvLocation];
		gotCSV = true;
		madeSelection = true;

	end
	run_update();

end

%callback for save location button
function saveButton_Callback(source,eventdata)

	madeSelection = false;
	while not(madeSelection),
		disp('Pick .mat file to save results to, or create a new one');

		if gotFolder,
			startFolder = folderName;
		else
			startFolder = pwd;
		end

		[matLocation,PathName,FilterIndex] = uiputfile([startFolder, '/randforest.mat']);
		if FilterIndex==0,
			break;
		end
		saveLocation = [PathName,matLocation];
		gotSaveLoc = true;
		madeSelection = true;

	end
	run_update();

end


%callback for algorithm button
function algorithmButton_Callback(source,eventdata)
 
	madeSelection = false;
	
		%initially there were going to be a set of standard localisation algorithm .csv layouts stored here to make things easier for the user. Cut out as this doesn't appear to be necessarily constant even if using the same algorithm (e.g. thunderstorm can change its number of rows). Force the user to specify by hand each time
	algorithm = 'Other';
        while not(madeSelection),
	        %must be other
	        prompt = {'How many columns does the .csv folder have?'};
	        dlg_title = 'Specify .csv 1/6';
	        num_lines = 1;
	        defAns = {num2str(csvNumCols)};
	        csvNumColsCell = inputdlg(prompt,dlg_title,num_lines,defAns);
		if length(csvNumColsCell)==0,
			break;
		end

	                
	        prompt = {'What column has the X co-ordinates?'};
	        dlg_title = 'Specify .csv 2/6';
	        num_lines = 1;
	        defAns = {num2str(csvXCol)};
	        csvXColCell = inputdlg(prompt,dlg_title,num_lines,defAns);
		if length(csvXColCell)==0,
			break;
		end
	        
                
	        prompt = {'What column has the Y co-ordinates?'};
	        dlg_title = 'Specify .csv 3/6';
	        num_lines = 1;
	        defAns = {num2str(csvYCol)};
	        csvYColCell = inputdlg(prompt,dlg_title,num_lines,defAns);
		if length(csvYColCell)==0,
			break;
		end
	        
	                
	        prompt = {'What column has the image number?'};
	        dlg_title = 'Specify .csv 4/6';
	        num_lines = 1;
	        defAns = {num2str(csvFCol)};
	        csvFColCell = inputdlg(prompt,dlg_title,num_lines,defAns); 
		if length(csvFColCell)==0,
			break;
		end
	        
	                
	        prompt = {'What is the offset of the localisations from the pixel co-ordinates? (if in doubt, assume 0)'};
	        dlg_title = 'Specify .csv 5/6';
	        num_lines = 1;
	        defAns = {num2str(pixelOffset)};
	        pixelOffsetCell = inputdlg(prompt,dlg_title,num_lines,defAns);
		if length(pixelOffsetCell)==0,
			break;
		end
	        
                		
	        coordTypes = {'nm', 'pixels'};
	        [s,v] = listdlg('PromptString','Localisation co-ords', 'SelectionMode','single', 'ListString',coordTypes);
		if length(s)==0,
			break;
		end

	        localisationsInNm = s==1;                
     		csvNumCols = str2num(csvNumColsCell{1});
	        csvXCol = str2num(csvXColCell{1});
		csvYCol = str2num(csvYColCell{1})';
		csvFCol = str2num(csvFColCell{1});
		pixelOffset = str2num(pixelOffsetCell{1});


		gotAlgo = true;
		madeSelection = true;

	end

	run_update();
end

%callback for number of samples to label button
function NLabelbutton_Callback(source,eventdata)


	if gotNLabel,%set default answer
		defAns = {num2str(NLabel)};
	else
		defAns = {'900'};
	end
	madeSelection = false;


	while not(madeSelection),
		disp('Set the number of samples to label');
		prompt = {'Enter number to sample:'};
		dlg_title = 'Set number to sample';
		num_lines = 1;
		answer = inputdlg(prompt,dlg_title,num_lines,defAns);
	
		if length(answer)==0,
			break;
		end

		if isempty(str2num(answer{1})),
			h = errordlg('Number of samples to label must be a number');
			uiwait(h);
		else
			madeSelection = true;
		end


		if madeSelection,
			NLabel = str2num(answer{1});
			if NLabel < 0,
				h = errordlg('Number of samples to label must be bigger than 0');
				uiwait(h);
				gotNLabel = false;
				madeSelection = false;
			else
				gotNLabel = true;
				madeSelection = true;
			end
		end
	end

	run_update();

end



%callback for button to indicate we're finished, and start training the tree
function finishedButton_Callback(source,eventdata)	

	close(gcf);
	run_training(folderName, imageExtension, pixelSize, csvLocation, algorithm, NLabel, numTrees, saveLocation, csvNumCols, csvXCol, csvYCol, csvFCol,pixelOffset,localisationsInNm);
end

%update function - run each time a value is updated, to check if we're got all the necessary information to begin. if we do, then the 'finished' button appears, allowing the user to begin labelling some samples
function run_update()
	if gotFolder,
		set(hfolderNameText, 'string', folderName);
		set(hfolderNameText, 'FontWeight', 'bold');

	end
	if gotImageExtension,
		set(himageExtensionText, 'String', imageExtension(2:end));
		set(himageExtensionText, 'FontWeight', 'bold');

	end

	if gotPixelSize,
		set(hpixelSizeText, 'String', num2str(pixelSize));
		set(hpixelSizeText, 'FontWeight', 'bold');

	end
	if gotCSV,
		set(hcsvLocationText, 'String', csvLocation);
		set(hcsvLocationText, 'FontWeight', 'bold');
		end

	if gotAlgo,
        	if strcmp(algorithm, 'Other'),
               		algorithmString = [num2str(csvNumCols), ' columns, X in column ', num2str(csvXCol), ', Y in column ', num2str(csvYCol), ', image number in column ', num2str(csvFCol), ', '];
                	algorithmString = [algorithmString, 'the localisations are in '];
                	if localisationsInNm,
                	    algorithmString = [algorithmString, 'nm'];
                	else
                	    algorithmString = [algorithmString, 'pixels'];
                	end
                	algorithmString = [algorithmString, ' and are offset by ', num2str(pixelOffset), ' pixels.'];  
                
                
                	set(halgorithmText, 'String', algorithmString);
            	else
                	set(halgorithmText, 'String', algorithm);
            	end
		set(halgorithmText, 'FontWeight', 'bold');
	end
	
	if gotNLabel,
		set(hNLabelText, 'String', num2str(NLabel));
		set(hNLabelText, 'FontWeight', 'bold');

	end

	if gotNumTrees,
		set(htreeText, 'String', num2str(numTrees));
		set(htreeText, 'FontWeight', 'bold');
	end

	if gotSaveLoc,
		set(hsaveText, 'String', num2str(saveLocation));
		set(hsaveText, 'FontWeight', 'bold');
	end



	if gotImageExtension&gotFolder,
		imageList = dir([folderName, '/', imageExtension]);
		if length(imageList) == 0,
			h = errordlg(['Whoa, I cant find a single image in ', folderName,' with the extension', imageExtension(2:end),'. You should check that.']);
			uiwait(h);
		end
	end

	if gotFolder&gotImageExtension&gotPixelSize&gotCSV&gotAlgo&gotNLabel&gotNumTrees&gotSaveLoc,
		hFinishedName    = uicontrol('Style','pushbutton','String','Start labelling','Position', [buttonOffset,hFinishedV,buttonWidth*2,buttonHeight], 'Callback',@finishedButton_Callback);
		set(hFinishedName,'Units', 'normalized');

	end
		
end



end
