function display_results(rcfd, predictClassTree, goodClass, badClass, tooDense)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Function to display the results of the classification, hopefully in a simple to understand format
%    Inputs:
%        rcfd - NX4 matrix, contains the row co-ord, column co-ord, frame number and directory number of each localisations
%        predictClassTree - MX1 vector, predictions of labels for Xpredict
%        goodClass - int, value indicating a good localisations
%        badClass - int, value indicating a too dense localisation
%        unknownClass - int, value indicating a background localisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rmax = max(rcfd(:,1));
cmax = max(rcfd(:,2));
rmin = min(rcfd(:,1));
cmin = min(rcfd(:,2));
scale = 5;
blur = 0.1;
blurkernel = fspecial('gaussian', 2*4*ceil(blur)+1, blur);
normaliseIntensities = false;


useLogIntensity = false;
goodImg = makeImg(rcfd(predictClassTree==goodClass, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);
badImg = makeImg(rcfd(predictClassTree==badClass, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);
denseImg = makeImg(rcfd(predictClassTree==tooDense, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);
fullImg = makeImg(rcfd(:, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);


scsz = get(0,'ScreenSize');
[Nrows Ncols] = size(fullImg);

%convert to pixel size
buttonWidth = 200;
buttonHeight = 25;
margin = 20;
textHeight = 20;
imgDispHeight = min(Nrows,scsz(4)/4);
imgDispWidth = min(Ncols,scsz(3)/4);
textWidth = imgDispWidth;


%set figure size
fheight = max(2*imgDispHeight+3*margin+2*textHeight, 7*buttonHeight+2*margin);
fwidth = 2*imgDispWidth+4*margin + buttonWidth;

fig = figure('Visible','on','Position',[scsz(1)+30,scsz(2)+30,fwidth,fheight]); 

colMaps = {parula, jet, hsv, hot, cool, spring, summer, autumn, winter, gray, bone, copper, pink, lines, colorcube, prism, flag, white};
colmapIndex = 4;

%create axes within figure
fullImgHcoord = margin;
fullImgVcoord = 2*margin+imgDispHeight+2*textHeight;
hfullImg = axes('Units','pixels','Position',[fullImgHcoord,fullImgVcoord,imgDispWidth,imgDispHeight]);
fullTextHcoord = fullImgHcoord;
fullTextVcoord = fullImgVcoord-textHeight;
hfullText = uicontrol('Style','text','String','Full Reconstruction','Position',[fullTextHcoord,fullTextVcoord,textWidth,textHeight]);


goodImgHcoord = 2*margin+imgDispWidth;
goodImgVcoord = 2*margin+imgDispHeight+2*textHeight;
hgoodImg = axes('Units','pixels','Position',[goodImgHcoord,goodImgVcoord,imgDispWidth,imgDispHeight]);
goodTextHcoord = goodImgHcoord;
goodTextVcoord = goodImgVcoord-textHeight;
hgoodText = uicontrol('Style','text','String','Accurate Reconstruction','Position',[goodTextHcoord,goodTextVcoord,textWidth,textHeight]);


badImgHcoord = margin;
badImgVcoord = margin+textHeight;
hbadImg = axes('Units','pixels','Position',[badImgHcoord,badImgVcoord,imgDispWidth,imgDispHeight]);
badTextHcoord = badImgHcoord;
badTextVcoord = badImgVcoord-textHeight;
hbadText = uicontrol('Style','text','String','Background Reconstruction','Position',[badTextHcoord,badTextVcoord,textWidth,textHeight]);



denseImgHcoord = 2*margin+imgDispWidth;
denseImgVcoord = margin+textHeight;
hdenseImg = axes('Units','pixels','Position',[denseImgHcoord,denseImgVcoord,imgDispWidth,imgDispHeight]);
denseTextHcoord = denseImgHcoord;
denseTextVcoord = denseImgVcoord-textHeight;
hdenseText = uicontrol('Style','text','String','High density Reconstruction','Position',[denseTextHcoord,denseTextVcoord,textWidth,textHeight]);


%buttons
hcolMapH = 2*imgDispWidth+3*margin;
hcolMapV = fheight-margin-buttonHeight;
hcolMapButton = uicontrol('Style','pushbutton','String','set color map','Position', [hcolMapH,hcolMapV,buttonWidth,buttonHeight], 'Callback',@colMapButton_Callback);

hlogH = hcolMapH;
hlogV = hcolMapV - buttonHeight;
hlogButton = uicontrol('Style','togglebutton','String','Use log intensities','Position', [hlogH,hlogV,buttonWidth,buttonHeight], 'Callback',@logIntensityButton_Callback);

hscaleH = hcolMapH;
hscaleV = hlogV - buttonHeight;
hscaleButton = uicontrol('Style','pushbutton','String','Set pixel upscaling','Position', [hscaleH,hscaleV,buttonWidth,buttonHeight], 'Callback',@scaleButton_Callback);


hblurH = hcolMapH;
hblurV = hscaleV - buttonHeight;
hblurButton = uicontrol('Style','pushbutton','String','Set Gaussian blur','Position', [hblurH,hblurV,buttonWidth,buttonHeight], 'Callback',@blurButton_Callback);

hnormaliseH = hcolMapH;
hnormaliseV = hblurV - buttonHeight;
hnormaliseButton = uicontrol('Style','togglebutton','String','Normalise intensities?','Position', [hnormaliseH,hnormaliseV,buttonWidth,buttonHeight], 'Callback',@normaliseButton_Callback);

hpopH = hcolMapH;
hpopV = hnormaliseV - buttonHeight;
hpopButton = uicontrol('Style','pushbutton','String','Pop out images','Position', [hpopH,hpopV,buttonWidth,buttonHeight], 'Callback',@popButton_Callback);

hquitH = hcolMapH;
hquitV = hpopV - buttonHeight;
hquitButton = uicontrol('Style','pushbutton','String','Quit','Position', [hquitH,hquitV,buttonWidth,buttonHeight], 'Callback',@quitButton_Callback);



%normalise coords
set(fig,'Units', 'normalized');
set(hfullImg,'Units', 'normalized');
set(hfullText,'Units', 'normalized');
set(hgoodImg,'Units', 'normalized');
set(hgoodText,'Units', 'normalized');
set(hbadImg,'Units', 'normalized');
set(hbadText,'Units', 'normalized');
set(hdenseImg,'Units', 'normalized');
set(hdenseText,'Units', 'normalized');
set(hcolMapButton,'Units', 'normalized');
set(hlogButton,'Units', 'normalized');
set(hscaleButton,'Units', 'normalized');
set(hblurButton,'Units', 'normalized');
set(hnormaliseButton,'Units', 'normalized');
set(hpopButton,'Units', 'normalized');
set(hquitButton,'Units', 'normalized');


carryOn = true;
while carryOn,
	
	goodImg = makeImg(rcfd(predictClassTree==goodClass, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);
	badImg = makeImg(rcfd(predictClassTree==badClass, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);
	denseImg = makeImg(rcfd(predictClassTree==tooDense, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);
	fullImg = makeImg(rcfd(:, 1:2), rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities);

	

	totalMin = min([fullImg(:); goodImg(:); badImg(:); denseImg(:)]);
	totalMax = max([fullImg(:); goodImg(:); badImg(:); denseImg(:)]);

	%show images
	hold(hfullImg, 'off'); 		imshow((fullImg-totalMin)/(totalMax-totalMin),  'Parent', hfullImg);	
	hold(hgoodImg, 'off'); 		imshow((goodImg-totalMin)/(totalMax-totalMin),  'Parent', hgoodImg);	
	hold(hbadImg, 'off'); 		imshow((badImg-totalMin)/(totalMax-totalMin),  'Parent', hbadImg);	
	hold(hdenseImg, 'off'); 	imshow((denseImg-totalMin)/(totalMax-totalMin),  'Parent', hdenseImg);	

	%set colourmap

	colormap(colMaps{colmapIndex});


	uiwait;
	%check if window has been closed
	carryOn = ishandle(fig) & carryOn;
	

end

%close fig if still exists (i.e. user quit rather than closing the window)
if ishandle(fig),
	close(fig);
end




	%%%%%%%%%%%%%%%%%%%
	%%%%%callbacks%%%%%
	%%%%%%%%%%%%%%%%%%%

	function quitButton_Callback(source,eventdata)
		carryOn = false;
		uiresume;
	end
	

	function popButton_Callback(source,eventdata)

		figure();	imshow((fullImg-totalMin)/(totalMax-totalMin));		colormap(colMaps{colmapIndex});	title('Full Reconstruction');
		figure();	imshow((goodImg-totalMin)/(totalMax-totalMin));		colormap(colMaps{colmapIndex});	title('Accurate Reconstruction');
		figure();	imshow((badImg-totalMin)/(totalMax-totalMin));		colormap(colMaps{colmapIndex});	title('Background Reconstruction');
		figure(); 	imshow((denseImg-totalMin)/(totalMax-totalMin));	colormap(colMaps{colmapIndex});	title('High density Reconstruction');

		figure(fig);
		uiresume;
	end



	function normaliseButton_Callback(source,eventdata)
		normaliseIntensities = get(hnormaliseButton, 'Value');
		uiresume;
	end



	function blurButton_Callback(source,eventdata)

		prompt = {'enter degree of gaussian blur'};
		dlg_title = 'Set blur';
		num_lines = 1;
		defAns = {num2str(blur)};
		answer = inputdlg(prompt,dlg_title,num_lines,defAns);


		if not(length(answer)==0),
			if not(isempty(str2num(answer{1}))),
				if(str2num(answer{1})>0)
					blur = str2num(answer{1});
					blurkernel = fspecial('gaussian', 2*4*ceil(blur)+1, blur);
				end
			end
		end

		uiresume;
	end



	function logIntensityButton_Callback(source,eventdata)


		useLogIntensity = get(hlogButton, 'Value');
		uiresume;
	end



	function scaleButton_Callback(source,eventdata)

		prompt = {'enter degree image is upscaled'};
		dlg_title = 'Set upscale';
		num_lines = 1;
		defAns = {num2str(scale)};
		answer = inputdlg(prompt,dlg_title,num_lines,defAns);

		if not(length(answer)==0),
			if not(isempty(str2num(answer{1}))),
				if(str2num(answer{1})>0)
					scale = str2num(answer{1});
					
				end
			end
		end


		uiresume;
	end



	function colMapButton_Callback(source,eventdata)
		mapsS = {'parula', 'jet', 'hsv', 'hot', 'cool', 'spring', 'summer', 'autumn', 'winter', 'gray', 'bone', 'copper', 'pink', 'lines', 'colorcube', 'prism', 'flag', 'white'};
		[s,v] = listdlg('PromptString','Select color map to display:', 'SelectionMode','single', 'ListString',mapsS);
		if v==0,
		

		else
			colmapIndex = s;
			colormap(colMaps{colmapIndex});
		end;
	end

	%%%%%%%%%%%%%%%%%%%
	%%%callbacks end%%%
	%%%%%%%%%%%%%%%%%%%
end





function img = makeImg(rc, rmax, rmin, cmax, cmin, scale, useLogIntensity, blurkernel, normaliseIntensities)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Makes an image, given parameters
%	Inputs:
%		rc - matrix, NX2, localisation co-ords
%		rmax - real, maximum of range of values of r
%		rmin - real, minimum of range of values of r
%		cmax - real, maximum of range of values of c
%		cmin - real, minimum of range of values of c
%		scale - real, degree rc is scaled prior to pixelation 
%		useLogIntensity - bool controls whether we use raw pixel values or log10(1+pixel) transform them
%		blurkernel - matrix, square, real valued, convolved with image histogram to add a bit of blur
%		normaliseIntensities - bool, controls whether intensities of image is normalised to range 0-1
%	Outputs:
%		img - matrix, real valued, image to be diplayed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	rc(:,1) = scale*(rc(:,1)-rmin+1);
	rc(:,2) = scale*(rc(:,2)-cmin+1);



	img = accumarray(ceil(rc),1,ceil(scale*([rmax-rmin,cmax-cmin]+1)));

	img = conv2(img, blurkernel, 'same');

	if useLogIntensity,
		img = log10(1+img);
	end


	if normaliseIntensities,
		img = img-min(img(:));
		img = img/max(img(:));
	end

end



