function run_training(folderName, imageExtension, pixelSize, csvFile, algorithm, NLabel, numTrees, saveLoc, csvNumCols, csvXCol, csvYCol, csvFCol,pixelOffset,localisationsInNm)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Called by START.m
%    After user has input details of where the raw data is saved, the localisations .csv file, etc, this function asks them to
%    label a subset of these localisations. It then trains the random forest classifier
%    Inputs:
%        folderName - string, Folder in which the raw images are stored
%        pixelSize - number, Pixel size in nm
%        csvFile - string, path to .csv file where the localisations are stored
%	 imageExtension - string, extension of image files, e.g. '.tiff'
%        algorithms - string, depreciated, used to auto-load .csvinformation for certain algorithms. Set to 'Other'.
%        ROI - matrix, depreciated, used to examine a sub-region of the image. Set to [].
%        csvNumCols - int, Total number of columns in the .csv file
%        csvXCol - int, Column number in which the x co-ords are stored in the .csv file
%        csvYCol - int, Column number in which the y co-ords are stored in the .csv file
%        csvFCol - int, Column number in which the frame numbers are stored in the .csv file
%        pixelOffset - number, offset between pixel and localisation co-ordinates
%        localisationsInNm - boolean, sets whether the localisations are stored in nm or pixel co-ords
%        NLabel - int, number of samples to be labelled
%        numTrees - int, number of trees in final classifier
%        saveLoc - string, location to save .mat file to
%        pixelOffset - number, offset of localisation co-ords raltive to pixel co-ords
%        localisationsInNm - bool, whether localisation positions are recorded in nm or not
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



goodClass = 1;							%basically an enumerate constant.
badClass = -1;
tooDense = 0;



folderPathList(1).name = folderName;
save(saveLoc);

%make features allin one go
disp('making features')
[features, rcfd, maxBright, minBright, meanFeature, V] = make_features_from_scratch(folderPathList, pixelSize, csvFile, imageExtension, algorithm, [], csvNumCols, csvXCol, csvYCol, csvFCol,pixelOffset,localisationsInNm);
save(saveLoc);

%label some of the samples.
disp('labelling')
[localisationIdx, assignedClass, imgDir, NLabel, doTraining] = label_samples(rcfd, imageExtension, folderPathList, goodClass, badClass, tooDense, NLabel, maxBright, minBright);

%if user did not tell us to quit, then we do the training and build a classifier
if doTraining,
	save(saveLoc);
	disp('training classifier')
	[accuracyTree, mdlTree, predictClassTree, posteriortree] =  train_tree(features(localisationIdx,:),assignedClass,features,numTrees);
	save(saveLoc);


	%some output to indicate we're done
	disp('Classifier trained and results saved. To view, load the .mat file you specified earlier into Matlab.' );
	disp('The following variables may be of particular interest:');
	disp('    *localisations are stored in rcfd - this gives the row co-ord, column co-ord, frame number in the sequence, and directory number the image sequence was taken from');
	disp('    *Estimated class of localisations is stored in predictClassTree. A 1 signifies a localisation felt to be accurate, a 0 one felt to be too dense, and a -1 otherwise');
	disp('    *An estimate of the classifier accuracy is stored in accuracyTree');
	disp(' ');
	disp('e.g. to plot all good localisations, you might call:');
	disp('>>include = predictClassTree==1;');
	disp('>>figure();');
	disp('>>plot(rcfd(include,2), rcfd(include,1), ''g.'')');



	display_results(rcfd, predictClassTree, goodClass, badClass, tooDense);


end




disp('Finished')


end













