function [accuracy, MDL, Ypredict, posterior] =  train_tree(Xtrain,Ytrain,Xpredict,numtree)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Called by run_training.m
%    Train a random forest classifier, and make predictions.
%    inputs:
%        Xtrain - NXD matrix, features to train on
%        Ytrain - NX1 vector, labels to train on
%        Xpredict - MXD matrix, features to make predictions over following training
%        numtree - int, how many trees to train in the forest
%    outputs:
%        accuracy - double, cross validated accuracy of classifier
%        MDL - random forest model, the trained model
%        Ypredict - MX1 vector, predictions of labels for Xpredict
%        posterior - matrix, predictions of posterior class probabilities for each Xpredict
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	h = waitbar(0,'Training tree');

	f = @(x,y,xt,yt)(fun(x,y,xt,yt,numtree));

	accuracy = mean(crossval(f, Xtrain, Ytrain));

	waitbar(1/3);
	
	MDL = TreeBagger(numtree,Xtrain, Ytrain);

	[YpredictCell, posterior, stddev] = predict(MDL, Xpredict);

	waitbar(2/3);

	Ypredict = zeros(size(Xpredict,1),1);

	for i=1:length(YpredictCell),
		Ypredict(i) = str2num(cell2mat(YpredictCell(i)));
	end

	close(h);
end



function a = fun(x,y,xt,yt,numtree)

%%%%%%%%%%%%%%%%
%
%    inputs:
%        x - NXD matrix, training features
%        y - NX1 vector, training labels
%        xt - MXD matrix, testing features
%        yt - MX1 vector, testing labels
%        numtree - int, number of trees in forest
%    outputs:
%        a - number, accuracy

	m = TreeBagger(numtree,x,y);

	ypc = predict(m,xt);
	yp = zeros(size(yt));
	for i=1:length(ypc),
		yp(i) = str2num(cell2mat(ypc(i)));
	end
	a = mean(double(yp==yt));
end
